TITLE PROG2.asm                         ;File name of program

LF equ 0ah                              ;ASCII 10 - newline character
CR equ 0dh                              ;ASCII 13 - carriage return character
NEWLINE equ CR,LF                       ;Combine CR and LF for carriage return
DOUBLESPC equ NEWLINE,LF                ;Combine NEWLINE and LF for double space
MAXSTR equ 256d                         ;Accept up to 256 character string

.586                                    ;Allow for Pentium instrucitons
.MODEL FLAT                             ;Memory model is FLAT (4GB)
INCLUDE io32.inc                        ;Include the 32 bit I/O procedures
.STACK  4096h                           ;4k hex bytes for stack
.DATA                                   ;Data segment begins here
   Intro      BYTE  NEWLINE,'WELCOME!',NEWLINE,\
                    'This program allows any user to enter ',\
                    'a binary, decimal, or hexadecimal number. The program ',\
                    'will then convert that number to the other two formats ',\
                    'and ',NEWLINE,\
                    'display the results on the screen.',DOUBLESPC,0
   Error1     BYTE  DOUBLESPC,'Error: Invalid input.',\
                    ' Please try again.',DOUBLESPC,0
   Error2     BYTE  DOUBLESPC,'Error: Input too large.',\
                    ' Please try again.',DOUBLESPC,0
   Menu       BYTE  NEWLINE,' 1) Enter a Binary Number',\
                    NEWLINE,' 2) Enter a Decimal Number',\
                    NEWLINE,' 3) Enter a Hexadecimal Number',\
                    NEWLINE,' 4) Quit',NEWLINE,NEWLINE,0
   Prompt     BYTE  NEWLINE,'Enter your choice --> ',0
   BPrompt    BYTE  NEWLINE,NEWLINE,'Enter a binary number ',\
                    '(less than 33 digits) --> ',0
   DPrompt    BYTE  NEWLINE,NEWLINE,'Enter a decimal number ',\
                    '(less than 4294967295d) --> ',0
   HPrompt    BYTE  NEWLINE,NEWLINE,'Enter a hexadecimal number ',\
                    '(less than 0FFFFFFFFh) --> ',0
   BinAns     BYTE  DOUBLESPC,'Binary=',0
   DecAns     BYTE  DOUBLESPC,'Decimal=',0
   HexAns     BYTE  DOUBLESPC,'Hexadecimal=0',0
   TempChar   BYTE  ?                     ;Temporary character storage
   BinVal     DWORD 0h                    ;Will hold final Binary Value
   DecVal     DWORD 0h                    ;Will hold final Decimal Value
   HexVal     DWORD 0h                    ;Will hold final Hex Value
   String     BYTE  MAXSTR dup(?)         ;Holds max sring size
   TempString BYTE  MAXSTR DUP(0)         ;Temporary string storage
   TempInt    DWORD 0h                    ;Temporary integer storage
   KeyPress   BYTE  ?                     ;Temporary storage
   TempNum    DWORD ?                     ;Temporary number storage
   Number     DWORD 0h                    ;Holds number value

;************************** Main Body of Program *****************************
; Given  : Nothing
; Process: Main Body - calls procedures to do the processing
; Return : Nothing
;*****************************************************************************
.CODE                                     ;executable section begins here
_main:
        lea  esi,Intro                    ;point to intro string in data
        call PrintString                  ;display the string
Repeat1:lea  esi,Menu                     ;point to menu string in data
        call PrintString                  ;display the string
        lea  esi,Prompt                   ;point to prompt message
        call PrintString                  ;display the string
        lea  esi,TempNum                  ;point to storage for keypressed
        call GetChar                      ;  and get the char
Check1: cmp  TempNum,'1'                  ;Test if 1 was typed
        jne  Check2                       ;  no, check for 2
        call Binary                       ;If number equals 1 go to binary
        jmp  Repeat1                      ;go to menu
Check2: cmp  TempNum,'2'                  ;Test if 2 was typed
        jne  Check3                       ;  no, check for 3
        call Decimal                      ;If number equals 2 go to Decimal
        jmp  Repeat1                      ;go to menu
Check3: cmp  TempNum,'3'                  ;Test if 3 was typed
        jne  Check4                       ;  no, check for 4
        call Hexadecimal                  ;If number equals 3 go to hexadecimal
        jmp  Repeat1                      ;go to menu
Check4: cmp  TempNum,'4'                  ;Test if 4 was typed
        je   EndProgram                   ;  yes, end the program
        call MenuError                    ;go to error message for other inputs
EndProgram:
        INVOKE  ExitProcess,0             ;Exit program with return code 0

;********** Procedure to Convert Binary to Decimal & Hexadecimal **************
; Given   : Menu option 1
; Process : Prompts user for binary number and calls 2 procedures to convert
;         : from binary to decimal and hexadecimal
; Return  : None
;******************************************************************************
Binary PROC NEAR32
        pushad                          ;Save the contents of all registers
        pushfd                          ;
        lea  esi,DPrompt                ;point to dprompt message
        call PrintString                ;display the string
        call GetNumBin                  ;Get user decimal number
;        cmp GetNumBin, 4294967295    ;compare getnum to max input
        cmp eax, 4294967295    ;compare getnum to max input
        jg   TooLargeError              ;if greater, go to TooLargeError
        call ConvertB2D                 ;call convertb2d for binary to decimal
        call ConvertB2H                 ;call convertb2h for binary to hex
        call Repeat1                    ;Go to Menu options
        popfd                           ;Restore the registers
        popad                           ;
        ret                             ;Return to Calling procedure
Binary ENDP

;********** Procedure to Convert Decimal to Binary & Hexadecimal **************
; Given   : Menu option 2
; Process : Prompts user for decimal number and calls 2 procedures to convert
;         : from decimal to binary and hexadecimal
; Return  : None
;******************************************************************************
Decimal PROC NEAR32
        pushad                          ;Save the contents of all registers
        pushfd                          ;
        lea  esi,DPrompt                ;point to dprompt message
        call PrintString                ;display the string
        call GetNumDec                  ;Get user decimal number
;        cmp  GetNumDec, 4294967295    ;compare getnum to max input
        cmp eax, 4294967295    ;compare getnum to max input
        jg   TooLargeError              ;if greater, go to TooLargeError
        call ConvertD2B                 ;call convertd2b for decimal to binary
        call ConvertD2H                 ;call convertd2h for decimal to hex
        call Repeat1                    ;Go to Menu options
        popfd                           ;Restore the registers
        popad                           ;
        ret                             ;Return to Calling procedure
Decimal ENDP

;********** Procedure to Convert Hexadecimal to Binary & Decimal **************
; Given   : Menu option 3
; Process : Prompts user for hexadecimal number and calls 2 procedures to
;         : convert from hexadecimal to binary and decimal
; Return  : None
;******************************************************************************
Hexadecimal PROC NEAR32
        pushad                          ;Save the contents of all registers
        pushfd                          ;
        lea  esi,HPrompt                ;point to hprompt
        call PrintString                ;display the string
        call GetNumHex                  ;Get user Hex number
;        cmp  GetNumHex, 4294967295    ;compare getnum to max input
        cmp eax, 4294967295    ;compare getnum to max input
        jg   TooLargeError              ;if greater, go to TooLargeError
        call ConvertH2B                 ;call converth2b for hex to binary
        call ConvertH2D                 ;call converth2d for hex to decimal
        call Repeat1                    ;Go to Menu options
        popfd                           ;Restore the registers
        popad                           ;
        ret                             ;Return to Calling procedure
Hexadecimal ENDP

;********* Procedure to Print an Error for Invalid Menu Selection *************
; Given   : Invalid menu option
; Process : Prints an error message and recalls the menu to display so the
;         : user can input a valid selection
; Return  : None
;******************************************************************************
MenuError PROC NEAR32
        pushad                          ;Save the contents of all registers
        pushfd                          ;
        lea  esi,Error1                 ;point to error1 message
        call PrintString                ;display the string
        call Repeat1                    ;loops back to start of program
        popfd                           ;Restore the registers
        popad                           ;
        ret                             ;Return to Calling procedure
MenuError ENDP

;********** Procedure to Print an Error for Invalid Binary Input **************
; Given   : Invalid menu option
; Process : Prints an error message and recalls the menu to display so the
;         : user can input a valid selection
; Return  : None
;******************************************************************************
TooLargeError PROC NEAR32
        pushad                          ;Save the contents of all registers
        pushfd                          ;
        lea  esi,Error2                 ;point to error2 message
        call PrintString                ;display the string
        call Repeat1                    ;loops back to start of program
        popfd                           ;Restore the registers
        popad                           ;
        ret                             ;Return to Calling procedure
TooLargeError ENDP

;***************** Procedure to Convert Binary to Decimal *********************
; Given   : User defined binary number
; Process : converts the binary number to decimal and prints the string
;         :
; Return  : None
;******************************************************************************
ConvertB2D PROC NEAR32
        pushad                          ;Save the contents of all registers
        pushfd                          ;
        push 0h                         ;push 0h onto stack
        mov  eax, GetNumBin              ;copy user number into eax
Loop1:  cmp  eax,30h                    ;loop condition comparison
        je   Print                      ;if equal, jump to print
        
Print:  lea  esi,DecAns                 ;point to decans
        call PrintString                ;display the string
        pop  edx                        ;display new user number(s)

        popfd                           ;Restore the registers
        popad                           ;
        ret                             ;Return to Calling procedure
ConvertB2D ENDP

;*************** Procedure to Convert Binary to Hexadecimal *******************
; Given   : User defined binary number
; Process : Converts the binary number to hexadecimal and prints the string
;         :
; Return  : None
;******************************************************************************
ConvertB2H PROC NEAR32
        pushad                          ;Save the contents of all registers
        pushfd                          ;
        push 0h                         ;push 0h onto stack
        mov  eax,GetNumBin              ;copy user number into eax
Loop1:  cmp  eax,30h                    ;loop condition comparison
        je   Print                      ;if equal, jump to print
        
Print:  lea  esi,HexAns                 ;point to hexans
        call PrintString                ;display the string
        pop  edx                        ;display new user number(s)

        popfd                           ;Restore the registers
        popad                           ;
        ret                             ;Return to Calling procedure
ConvertB2H ENDP

;***************** Procedure to Convert Decimal to Binary *********************
; Given   : User defined decimal number
; Process : Converts the decimal number to a binary number
;         : and prints the string
; Return  : None
;******************************************************************************
ConvertD2B PROC NEAR32
        pushad                          ;Save the contents of all registers
        pushfd                          ;
        push 0h                         ;push 0h onto stack
        mov  ebx,2d                     ;copy 2d into ebx register
        mov  eax,GetNumDec              ;copy user number into eax
Loop1:  cmp  eax,30h                    ;loop condition comparison
        je   Print                      ;if equal, jump to print
        div  ebx                        ;divide eax register by 2
        push edx                        ;push edx onto stack
        jmp  Loop1                      ;call loop1 to process next number
Print:  lea  esi,BinAns                 ;point to binans
        call PrintString                ;display the string
        pop  edx                        ;display new user number(s)
Loop2:  cmp  edx,0h                     ;compare edx register to 0h
        je   Done                       ;  if equal, go to done
        mov  BinVal,edx                 ;copy edx into binval
        lea  esi,BinVal                 ;point to binval
        call PutChar                    ;display the character
        pop  edx                        ;display new user number(s)
        jmp  Loop2                      ;go to loop2 to process next number
Done:   popfd                           ;Restore the registers
        popad                           ;
        ret                             ;Return to Calling procedure
ConvertD2B ENDP

;*************** Procedure to Convert Decimal to Hexadecimal ******************
; Given   : User defined decimal number
; Process : Converts the decimal number to a hexadecimal number
;         : and prints the string
; Return  : None
;******************************************************************************
ConvertD2H PROC NEAR32
        pushad                          ;Save the contents of all registers
        pushfd                          ;
        push 0h                         ;push 0h onto stack
        mov  ebx,16d                    ;copy 16d into ebx register
        mov  eax,GetNumDec              ;copy user number into eax
Loop1:  cmp  eax,30h                    ;loop condition comparison
        je   Print                      ;if equal, jump to print
        div  ebx                        ;divide eax register by 16
        cmp  edx,39h                    ;compare cdx to 39h
        jg   Convert                    ;if greater, go to convert
        push edx                        ;  else, push edx to stack
Convert:cmp  edx,41h                    ;compare edx to 41h
        je   Hex_A                      ;  if equal, go to hex_a
        cmp  edx,42h                    ;compare edx to 42h
        je   Hex_B                      ;  if equal, go to hex_b
        cmp  edx,43h                    ;compare edx to 43h
        je   Hex_C                      ;  if equal, go to hex_c
        cmp  edx,44h                    ;compare edx to 44h
        je   Hex_D                      ;  if equal, go to hex_d
        cmp  edx,45h                    ;compare edx to 45h
        je   Hex_E                      ;  if equal, go to hex_e
        cmp  edx,46h                    ;compare edx to 46h
        je   Hex_F                      ;  if equal, go to hex_f
Hex_A:  push 41h                        ;push 41h on the stack
        jmp  Loop1                      ;call loop1 to process next number
Hex_B:  push 42h                        ;push 42h on the stack
        jmp  Loop1                      ;call loop1 to process next number
Hex_C:  push 43h                        ;push 43h on the stack
        jmp  Loop1                      ;call loop1 to process next number
Hex_D:  push 44h                        ;push 44h on the stack
        jmp  Loop1                      ;call loop1 to process next number
Hex_E:  push 45h                        ;push 45h on the stack
        jmp  Loop1                      ;call loop1 to process next number
Hex_F:  push 46h                        ;push 46h on the stack
        jmp  Loop1                      ;call loop1 to process next number
Print:  lea  esi,HexAns                 ;point to hexans
        call PrintString                ;display the string
        pop  edx                        ;display new user number(s)
Loop2:  cmp  edx,0h                     ;compare edx to 0h
        je   Done                       ;if equal to 0h, go to done
        mov  HexVal,edx                 ;copy edx value into hexval
        lea  esi,HexVal                 ;point to hexval
        call PutChar                    ;display the character
        pop  edx                        ;display new user number(s)
        jmp  Loop2                      ;call loop2 to process next number
Done:   popfd                           ;Restore the registers
        popad                           ;
        ret                             ;Return to Calling procedure
ConvertD2H ENDP

;*************** Procedure to Convert Hexadecimal to Binary *******************
; Given   : User defined hexadecimal number
; Process : Converts the hexadecimal number to binary
;         : and prints the string
; Return  : None
;******************************************************************************
ConvertH2B PROC NEAR32
        pushad                          ;Save the contents of all registers
        pushfd                          ;
        push 0h                         ;push 0h onto stack
        mov  eax,GetNumHex              ;copy user number into eax
Loop1:  cmp  eax,30h                    ;loop condition comparison
        je   Print                      ;if equal, go to print
        
Print:  lea  esi,BinAns                 ;point to binans
        call PrintString                ;display the string
        pop  edx                        ;display new user number(s)

        popfd                           ;Restore the registers
        popad                           ;
        ret                             ;Return to Calling procedure
ConvertH2B ENDP

;*************** Procedure to Convert Hexadecimal to Decimal ******************
; Given   : User defined hexadecimal number
; Process : Converts the hexadecimal number to decimal
;         : and prints the string
; Return  : None
;******************************************************************************
ConvertH2D PROC NEAR32
        pushad                          ;Save the contents of all registers
        pushfd                          ;
        push 0h                         ;push 0h onto stack
        mov  eax,GetNumHex              ;copy user number into eax
Loop1:  cmp  eax,30h                    ;loop condition comparison
        je   Print                      ;if equal, go to print
        
Print:  lea  esi,DecAns                 ;point to decans
        call PrintString                ;display the string
        pop  edx                        ;display new user number(s)

        popfd                           ;Restore the registers
        popad                           ;
        ret                             ;Return to Calling procedure
ConvertH2D ENDP

;**************** Procedure to ??????????????????????????? ********************
; Given   :
; Process :
;         :
; Return  :
;******************************************************************************
MyProc PROC NEAR32
        pushad                          ;Save the contents of all registers
        pushfd                          ;

        popfd                           ;Restore the registers
        popad                           ;
        ret                             ;Return to Calling procedure
MyProc ENDP

;******************* Near procedure to print a string *************************
; This procedure uses a  WHILE... loop to print a string of characters
; Given   :  The Address of the String to print on the STACK
; Process :  Print the String by repeated calls to PrintChar procedure
; Return  :  Nothing
;******************************************************************************
MyPrintString PROC NEAR32               ;Define a NEAR32 procedure
        pop eax                         ;Get RETURN address from Stack
        pop ebx                         ;Get STRING address from Stack
        push eax                        ;Put RETURN address back on Stack
While1: mov dl,[ebx]                    ;Contents of address pointed to by EBX
        mov TempChar,dl                 ;Move the character to temporary store
        cmp TempChar,0                  ;While not end of the string
        je StringEnd                    ;  Jump to StringEnd on char = 0 (null)
        lea esi,TempChar                ;Address of character to print
        call PutChar                    ;Go print a character
        inc ebx                         ;Increment pointer to next character
        jmp While1                      ;Go to top of the loop
StringEnd:ret                           ;Return to calling procedure
MyPrintString ENDP                      ;Formal end of procedure

;********************** Procedure to Input the Number *************************
; Given   :  Nothing
; Process :  Accept a string of ASCII digits and convert them to an integer
; Return  :  Return the integer in EAX register
;******************************************************************************
GetNumBin PROC NEAR32                    ;Define a NEAR32 procedure
        push ebx                         ;Save the contents of all registers
        push ecx                         ;  except for EAX, which will contain 
        push edx                         ;  the integer number that was read
        lea esi,TempString               ;Pointer to temporary string
        call GetString                   ;  go get the string
        lea esi,TempString               ;Pointer to temporary string
        lea edi,TempInt                  ;Pointer to integer storage
        call Ascii2Bin                   ;Convert the string to integer
        mov eax,TempInt                  ;Copy the integer to EAX
        pop edx                          ;Restore the registers in reverse order
        pop ecx                          ;
        pop ebx                          ;
        ret                              ;Return to Calling procedure
GetNumBin ENDP                           ;Formal end of procedure

;********************** Procedure to Input the Number *************************
; Given   :  Nothing
; Process :  Accept a string of ASCII digits and convert them to an integer
; Return  :  Return the integer in EAX register
;******************************************************************************
GetNumDec PROC NEAR32                    ;Define a NEAR32 procedure
        push ebx                         ;Save the contents of all registers
        push ecx                         ;  except for EAX, which will contain 
        push edx                         ;  the integer number that was read
        lea esi,TempString               ;Pointer to temporary string
        call GetString                   ;  go get the string
        lea esi,TempString               ;Pointer to temporary string
        lea edi,TempInt                  ;Pointer to integer storage
        call Ascii2Dec                   ;Convert the string to integer
        mov eax,TempInt                  ;Copy the integer to EAX
        pop edx                          ;Restore the registers in reverse order
        pop ecx                          ;
        pop ebx                          ;
        ret                              ;Return to Calling procedure
GetNumDec ENDP                           ;Formal end of procedure

;********************** Procedure to Input the Number *************************
; Given   :  Nothing
; Process :  Accept a string of ASCII digits and convert them to an integer
; Return  :  Return the integer in EAX register
;******************************************************************************
GetNumHex PROC NEAR32                    ;Define a NEAR32 procedure
        push ebx                         ;Save the contents of all registers
        push ecx                         ;  except for EAX, which will contain 
        push edx                         ;  the integer number that was read
        lea esi,TempString               ;Pointer to temporary string
        call GetString                   ;  go get the string
        lea esi,TempString               ;Pointer to temporary string
        lea edi,TempInt                  ;Pointer to integer storage
        call Ascii2Hex                   ;Convert the string to integer
        mov eax,TempInt                  ;Copy the integer to EAX
        pop edx                          ;Restore the registers in reverse order
        pop ecx                          ;
        pop ebx                          ;
        ret                              ;Return to Calling procedure
GetNumHex ENDP                           ;Formal end of procedure

;**************** Procedure to Convert ASCII to Integer ***********************
; Given   :  A pointer to the string to be Converted in the ESI register and a
;         :  pointer to the destination integer in EDI
; Process :  Convert the string of ASCII digits to an integer and store the
;         :  string in the address pointed to by EDI.  No registers are changed
;         :  and the flags are not affected.
; Return  :  Nothing
;******************************************************************************
Ascii2Bin PROC NEAR32
        pushad                          ;Save the contents of all registers
        pushfd                          ;
        cld                             ;Set the direction flag for forward
        mov ebx,0h                      ;Zero the EBX register
        xor eax,eax                     ;Clear the EAX register
NextIn: lodsb                           ;Get a character from the string
        cmp al,'0'                      ;If the character is less than 0, then
        jb Done                         ;  we have all the number and are Done
        cmp al,'9'                      ;If the character is more than 9, then
        ja Done                         ;  we have all the number and are Done
        and al,0Fh                      ;Mask out the high 4 bits, converting to Int
        xor ah,ah                       ;Zero the high byte of AX
        push eax                        ;Save the digit on the stack
        mov eax,2d                     ;Place 10 decimal in AX to multiply by
        mul ebx                         ;Multiply the number by 10
        mov ebx,eax                     ;Get number from EAX and put in EBX
        pop eax                         ;Get the digit back from the stack
        add ebx,eax                     ;Add the digit to the number
        jmp NextIn                      ;Get the next digit
Done:   mov [edi],ebx                   ;Save NUMBER at address pointed to by EDI
        popfd                           ;Restore the registers
        popad                           ;
        ret                             ;Return to Calling procedure
Ascii2Bin ENDP

;**************** Procedure to Convert Integer to ASCII ***********************
; Given   :  A pointer to the integer to be converted in the ESI register and a
;            pointer to the output string in EDI
; Process :  Convert the integer to a string of ASCII digits and store the
;         :  string in the address pointed to by EDI.  No registers are changed
;         :  and the flags are not affected.
; Return  :  Nothing
;******************************************************************************
Bin2Ascii PROC NEAR32
        pushad                          ;Save the contents of all registers
        pushfd                          ;
        cld                             ;Set the direction flag for forward
        mov eax,[esi]                   ;copy the integer to eax
        mov ecx,0h                      ;Zero the CX register for digit counter
        mov ebx,2d                     ;Set up divisor of 10 decimal
NextOut:mov edx,0h                      ;Zero EDX reg for high order dword of div
        div ebx                         ;Divide number in EAX by 10d
        push edx                        ;Save remainder on the stack
        inc ecx                         ;Count the digit
        cmp eax,0h                      ;Is number in EAX greater than 0
        ja NextOut                      ;Yes, get next digit
CharOut:pop eax                         ;Get number from the stack
        or eax, 0030h                   ;Convert Int to Ascii by adding 30h
        stosb                           ;  and store  in the destination string
        dec ecx                         ;Reduce characters to print by one
        jnz CharOut                     ;If CX > 0 loop to print next digit
        mov al,0                        ;Place a NULL in AL
        stosb                           ;  and null terminate the string
        popfd                           ;Restore the registers
        popad                           ;
        ret                             ;Return to Calling procedure
Bin2Ascii ENDP

;**************** Procedure to Convert ASCII to Integer ***********************
; Given   :  A pointer to the string to be Converted in the ESI register and a
;         :  pointer to the destination integer in EDI
; Process :  Convert the string of ASCII digits to an integer and store the
;         :  string in the address pointed to by EDI.  No registers are changed
;         :  and the flags are not affected.
; Return  :  Nothing
;******************************************************************************
Ascii2Dec PROC NEAR32
        pushad                          ;Save the contents of all registers
        pushfd                          ;
        cld                             ;Set the direction flag for forward
        mov ebx,0h                      ;Zero the EBX register
        xor eax,eax                     ;Clear the EAX register
NextIn: lodsb                           ;Get a character from the string
        cmp al,'0'                      ;If the character is less than 0, then
        jb Done                         ;  we have all the number and are Done
        cmp al,'9'                      ;If the character is more than 9, then
        ja Done                         ;  we have all the number and are Done
        and al,0Fh                      ;Mask out the high 4 bits, converting to Int
        xor ah,ah                       ;Zero the high byte of AX
        push eax                        ;Save the digit on the stack
        mov eax,10d                     ;Place 10 decimal in AX to multiply by
        mul ebx                         ;Multiply the number by 10
        mov ebx,eax                     ;Get number from EAX and put in EBX
        pop eax                         ;Get the digit back from the stack
        add ebx,eax                     ;Add the digit to the number
        jmp NextIn                      ;Get the next digit
Done:   mov [edi],ebx                   ;Save NUMBER at address pointed to by EDI
        popfd                           ;Restore the registers
        popad                           ;
        ret                             ;Return to Calling procedure
Ascii2Dec ENDP

;**************** Procedure to Convert Integer to ASCII ***********************
; Given   :  A pointer to the integer to be converted in the ESI register and a
;            pointer to the output string in EDI
; Process :  Convert the integer to a string of ASCII digits and store the
;         :  string in the address pointed to by EDI.  No registers are changed
;         :  and the flags are not affected.
; Return  :  Nothing
;******************************************************************************
Dec2Ascii PROC NEAR32
        pushad                          ;Save the contents of all registers
        pushfd                          ;
        cld                             ;Set the direction flag for forward
        mov eax,[esi]                   ;copy the integer to eax
        mov ecx,0h                      ;Zero the CX register for digit counter
        mov ebx,10d                     ;Set up divisor of 10 decimal
NextOut:mov edx,0h                      ;Zero EDX reg for high order dword of div
        div ebx                         ;Divide number in EAX by 10d
        push edx                        ;Save remainder on the stack
        inc ecx                         ;Count the digit
        cmp eax,0h                      ;Is number in EAX greater than 0
        ja NextOut                      ;Yes, get next digit
CharOut:pop eax                         ;Get number from the stack
        or eax, 0030h                   ;Convert Int to Ascii by adding 30h
        stosb                           ;  and store  in the destination string
        dec ecx                         ;Reduce characters to print by one
        jnz CharOut                     ;If CX > 0 loop to print next digit
        mov al,0                        ;Place a NULL in AL
        stosb                           ;  and null terminate the string
        popfd                           ;Restore the registers
        popad                           ;
        ret                             ;Return to Calling procedure
Dec2Ascii ENDP

;**************** Procedure to Convert ASCII to Integer ***********************
; Given   :  A pointer to the string to be Converted in the ESI register and a
;         :  pointer to the destination integer in EDI
; Process :  Convert the string of ASCII digits to an integer and store the
;         :  string in the address pointed to by EDI.  No registers are changed
;         :  and the flags are not affected.
; Return  :  Nothing
;******************************************************************************
Ascii2Hex PROC NEAR32
        pushad                          ;Save the contents of all registers
        pushfd                          ;
        cld                             ;Set the direction flag for forward
        mov ebx,0h                      ;Zero the EBX register
        xor eax,eax                     ;Clear the EAX register
NextIn: lodsb                           ;Get a character from the string
        cmp al,'0'                      ;If the character is less than 0, then
        jb Done                         ;  we have all the number and are Done
        cmp al,'9'                      ;If the character is more than 9, then
        ja Done                         ;  we have all the number and are Done
        and al,0Fh                      ;Mask out the high 4 bits, converting to Int
        xor ah,ah                       ;Zero the high byte of AX
        push eax                        ;Save the digit on the stack
        mov eax,16d                     ;Place 10 decimal in AX to multiply by
        mul ebx                         ;Multiply the number by 10
        mov ebx,eax                     ;Get number from EAX and put in EBX
        pop eax                         ;Get the digit back from the stack
        add ebx,eax                     ;Add the digit to the number
        jmp NextIn                      ;Get the next digit
Done:   mov [edi],ebx                   ;Save NUMBER at address pointed to by EDI
        popfd                           ;Restore the registers
        popad                           ;
        ret                             ;Return to Calling procedure
Ascii2Hex ENDP

;**************** Procedure to Convert Integer to ASCII ***********************
; Given   :  A pointer to the integer to be converted in the ESI register and a
;            pointer to the output string in EDI
; Process :  Convert the integer to a string of ASCII digits and store the
;         :  string in the address pointed to by EDI.  No registers are changed
;         :  and the flags are not affected.
; Return  :  Nothing
;******************************************************************************
Hex2Ascii PROC NEAR32
        pushad                          ;Save the contents of all registers
        pushfd                          ;
        cld                             ;Set the direction flag for forward
        mov eax,[esi]                   ;copy the integer to eax
        mov ecx,0h                      ;Zero the CX register for digit counter
        mov ebx,16d                     ;Set up divisor of 10 decimal
NextOut:mov edx,0h                      ;Zero EDX reg for high order dword of div
        div ebx                         ;Divide number in EAX by 10d
        push edx                        ;Save remainder on the stack
        inc ecx                         ;Count the digit
        cmp eax,0h                      ;Is number in EAX greater than 0
        ja NextOut                      ;Yes, get next digit
CharOut:pop eax                         ;Get number from the stack
        or eax, 0030h                   ;Convert Int to Ascii by adding 30h
        stosb                           ;  and store  in the destination string
        dec ecx                         ;Reduce characters to print by one
        jnz CharOut                     ;If CX > 0 loop to print next digit
        mov al,0                        ;Place a NULL in AL
        stosb                           ;  and null terminate the string
        popfd                           ;Restore the registers
        popad                           ;
        ret                             ;Return to Calling procedure
Hex2Ascii ENDP

Public _main
END _main                                     ;Code segment ends

OPT_Tmp2Asm	1






