;*****************************************************************************
; This include file is designed to be included with programs that need access
; to the console via the kernel32.lib and therefore kernel32.lib must be
; included (make sure it is in your path).  The procedures provided are:
;          GetChar (parameter passed in esi)
;          PutChar (parameter passed in esi)
;          GetString (parameter passed in esi and requires MAXSTR constant)
;          PrintString (parameter passed in esi)
;          Ascii2Int (parameters passed in esi and edi)
;          Int2Ascii (parameters passed in esi and edi)
; See Procedure headers for additional details
;
; Written by Joe Toone 10/29/2003 2:10:51 PM
;            Modified 2/16/2004 11:16:05 AM (removed unused variable)
;            Modified 10/25/2004 10:28:07 AM (converted from signed to unsigned
;                                             jumps in Ascii2Int and Int2Ascii)
;            Modified 2/20/2008 11:14:14 AM (added cld to PrintString)
;*****************************************************************************
.NOLIST                                 ;turn off listing so only the source
                                        ;code being written is in the .lst file
ENABLE_PROCESSED_INPUT equ 1            ;Flag to turn off line buffering
ENABLE_PROCESSED_OUTPUT equ 1           ;Flag to turn off line bufferin
ENABLE_LINE_WRAP equ 3                  ;Flag to trun line wrap on
DISABLE_PROCESSED_INPUT equ 7           ;Flag to turn on line buffering
STD_INPUT  equ -10d                     ;Function number for keyboard input
STD_OUTPUT equ -11d                     ;Function number for monitor output

.586                                    ;Allow for Pentium instrucitons

INCLUDELIB \masm32\lib\kernel32.lib                 ;Include the kernel 32 library

ExitProcess PROTO NEAR32 stdcall,       ;Prototype for Exit of OS
    dwExitCode:DWORD

SetConsoleMode PROTO NEAR32 stdcall,    ;Prototype for attaching to console
    hConsoleHandle:DWORD, dwMode:DWORD

GetStdHandle PROTO NEAR32 stdcall,      ;Prototype for obtaining a file handle
    nStdHandle:DWORD

ReadFile PROTO NEAR32 stdcall,          ;Prototype for reading a file
    hFile:DWORD, lpBuffer:NEAR32,
    nNumberOfCharsToRead:DWORD,
    lpNumberOfBytesRead:NEAR32,
    lpOverlapped:NEAR32

WriteFile PROTO NEAR32 stdcall,         ;Prototype for writing a file
    hFile:DWORD, lpBuffer:NEAR32,
    nNumberOfCharsToWrite:DWORD,
    lpNumberOfBytesWritten:NEAR32,
    lpOverlapped:NEAR32

GetLocalTime PROTO NEAR32 stdcall,       ;Prototype to get system time
    lpSystemTime:NEAR32

.DATA                                   ;Local Data segment begins here
   strAddr    DWORD  ?
   strLength  DWORD  ?
   hStdOut    DWORD  ?
   hStdIn     DWORD  ?
   read       DWORD  ?
   written    DWORD  ?

.CODE                                   ;executable section begins here
;********************* Near procedure to get a Character **********************
; Given   :  The Address of the Character to get in ESI register
; Process :  Input the Character using the kernel32.lib ReadFile from the
;         :  Standard_Input function call.  No registers are changed and the
;         :  flags are not affected.
; Return  :  The input character in the data segment
;******************************************************************************
GetChar PROC NEAR32                         ; Define procedure
            pushad                          ; save all registers
            pushfd                          ; save flags
            INVOKE GetStdHandle,STD_INPUT   ; get handle for keyboard
            mov hStdIn, eax                 ; save the handle
            INVOKE SetConsoleMode,          ; invoke standard console with
            hStdIn,                         ;   file handle for keyboard
            ENABLE_PROCESSED_INPUT          ;   turn line buffering off
            INVOKE ReadFile,                ; invoke standard ReadFile with
              hStdIn,                       ;   file handle for keyboard
              esi,                          ;   address of character
              1,                            ;   length of one byte
              NEAR32 PTR read,              ;   variable for # bytes read
              0                             ;   overlapped mode
            call PutChar                    ; echo the character on screen
            popfd                           ; restore flags
            popad                           ; restore registers
            ret                             ; return to caller
GetChar   ENDP

;******************* NEAR32 procedure to print a Character ********************
; Given   :  The Address of the Character to print in ESI register
; Process :  Print the Character using the kernel32.lib WriteFile to
;         :  Standard_Output function call.  No registers are changed and the
;         :  flags are not affected.
; Return  :  Nothing
;******************************************************************************
PutChar PROC NEAR32                         ; Define procedure
            pushad                          ; save registers
            pushfd                          ; save flags
            INVOKE GetStdHandle,STD_OUTPUT  ; get handle for console output
            mov hStdOut, eax                ; copy file handle for screen
	    INVOKE SetConsoleMode,          ; invoke standard console with
            hStdOut,                        ;   file handle for screen
            ENABLE_PROCESSED_OUTPUT	    ;   turn line buffering off
	    INVOKE WriteFile,               ; invoke standard WriteFile with
            hStdOut,                        ;   file handle for screen
            esi,                            ;   address of character
            1,                              ;   length of one byte
            NEAR32 PTR written,             ;   variable for # bytes written
            0                               ;   overlapped mode
            popfd                           ; restore flags
            popad                           ; restore registers
            ret                             ; return to caller
PutChar ENDP

;********************* Near procedure to get a String *************************
; Given   :  The Address of the String to fill in ESI register
; Process :  Input the String using the kernel32.lib ReadFile from the
;         :  Standard_Input function call.  No registers are changed and the
;         :  flags are not affected.
; Return  :  The input string in the data segment
;******************************************************************************
GetString PROC NEAR32                  ; Define procedure
            pushad                     ; save all registers
            pushfd                     ; save flags


            INVOKE GetStdHandle,STD_OUTPUT  ; get handle for console
            mov    hStdOut, eax             ; save the handle
            INVOKE SetConsoleMode,          ; invoke standard console with
            hStdOut,                        ;   file handle for keyboard
            ENABLE_LINE_WRAP                ;   turn line wrap on



            INVOKE GetStdHandle,STD_INPUT  ; get handle for console
            mov    hStdIn, eax         ; save the handle
            INVOKE SetConsoleMode,     ; invoke standard console with
            hStdIn,                    ;   file handle for keyboard
            DISABLE_PROCESSED_INPUT    ;   turn line buffering on
            mov    ecx, MAXSTR         ; string length
            mov    strLength, ecx      ; maximum string to accept
            mov    strAddr, esi        ; save pointer to input string
            INVOKE ReadFile,           ; invoke standard ReadFile with
              hStdIn,                  ;   file handle for keyboard
              strAddr,                 ;   address of string
              strLength,               ;   length of string
              NEAR32 PTR read,         ;   variable for # bytes read
              0                        ;   overlapped mode
            mov ecx, read              ; number of bytes read
            mov BYTE PTR [esi+ecx-2],0 ; replace CR/LF by trailing null
            popfd                      ; restore flags
            popad                      ; restore registers
            ret                        ; return to caller
GetString   ENDP

;******************* NEAR32 procedure to print a string ***********************
; Given   :  The Address of Null (0) terminated String to print in ESI register
; Process :  Print the String using the kernel32.lib WriteFile to
;         :  Standard_Output function call.  No registers are changed and the
;         :  flags are not affected.
; Return  :  Nothing
;******************************************************************************
PrintString PROC   NEAR32
            pushad                     ; save registers
            pushfd                     ; save flags
            cld                        ; required to print forward
            mov    strAddr, esi        ; copy string address
                                       ; find string length
            mov    strLength, 0        ; initialize string length
WhileChar:  cmp    BYTE PTR [esi], 0   ; character = null?
            jz     EndWhileChar        ; exit if so
            inc    strLength           ; increment character count
            inc    esi                 ; point at next character
            jmp    WhileChar           ; while more characters exist
EndWhileChar:
            INVOKE GetStdHandle,STD_OUTPUT ; get handle for console output
            mov    hStdOut, eax        ; copy file handle for screen
            INVOKE WriteFile,          ; invoke standard WriteFile with
              hStdOut,                 ;   file handle for screen
              strAddr,                 ;   address of string
              strLength,               ;   length of string
              NEAR32 PTR written,      ;   variable for # bytes written
              0                        ;   overlapped mode
            popfd                      ; restore flags
            popad                      ; restore registers
            ret                        ; return to caller
PrintString ENDP
;**************** Procedure to Convert ASCII to Integer ***********************
; Given   :  A pointer to the string to be Converted in the ESI register and a
;         :  pointer to the destination integer in EDI
; Process :  Convert the string of ASCII digits to an integer and store the
;         :  string in the address pointed to by EDI.  No registers are changed
;         :  and the flags are not affected.
; Return  :  Nothing
;******************************************************************************
Ascii2Int PROC NEAR32
        pushad                          ;Save the contents of all registers
        pushfd                          ;
        cld                             ;Set the direction flag for forward
        mov ebx,0h                      ;Zero the EBX register
        xor eax,eax                     ;Clear the EAX register
NextIn: lodsb                           ;Get a character from the string
        cmp al,'0'                      ;If the character is less than 0, then
        jb Done                         ;  we have all the number and are Done
        cmp al,'9'                      ;If the character is more than 9, then
        ja Done                         ;  we have all the number and are Done
        and al,0Fh                      ;Mask out the high 4 bits, converting to Int
        xor ah,ah                       ;Zero the high byte of AX
        push eax                        ;Save the digit on the stack
        mov eax,10d                     ;Place 10 decimal in AX to multiply by
        mul ebx                         ;Multiply the number by 10
        mov ebx,eax                     ;Get number from EAX and put in EBX
        pop eax                         ;Get the digit back from the stack
        add ebx,eax                     ;Add the digit to the number
        jmp NextIn                      ;Get the next digit
Done:   mov [edi],ebx                   ;Save NUMBER at address pointed to by EDI
        popfd                           ;Restore the registers
        popad                           ;
        ret                             ;Return to Calling procedure
Ascii2Int ENDP

;**************** Procedure to Convert Integer to ASCII ***********************
; Given   :  A pointer to the integer to be converted in the ESI register and a
;            pointer to the output string in EDI
; Process :  Convert the integer to a string of ASCII digits and store the
;         :  string in the address pointed to by EDI.  No registers are changed
;         :  and the flags are not affected.
; Return  :  Nothing
;******************************************************************************
Int2Ascii PROC NEAR32
        pushad                          ;Save the contents of all registers
        pushfd                          ;
        cld                             ;Set the direction flag for forward
        mov eax,[esi]                   ;copy the integer to eax
        mov ecx,0h                      ;Zero the CX register for digit counter
        mov ebx,10d                     ;Set up divisor of 10 decimal
NextOut:mov edx,0h                      ;Zero EDX reg for high order dword of div
        div ebx                         ;Divide number in EAX by 10d
        push edx                        ;Save remainder on the stack
        inc ecx                         ;Count the digit
        cmp eax,0h                      ;Is number in EAX greater than 0
        ja NextOut                      ;Yes, get next digit
CharOut:pop eax                         ;Get number from the stack
        or eax, 0030h                   ;Convert Int to Ascii by adding 30h
        stosb                           ;  and store  in the destination string
        dec ecx                         ;Reduce characters to print by one
        jnz CharOut                     ;If CX > 0 loop to print next digit
        mov al,0                        ;Place a NULL in AL
        stosb                           ;  and null terminate the string
        popfd                           ;Restore the registers
        popad                           ;
        ret                             ;Return to Calling procedure
Int2Ascii ENDP

.LIST                                   ;begin listing source code again
